@@ -10,8 +10,7 @@ import json
 import os
 import sqlite3
 from contextlib import contextmanager
-from typing import Any, Dict, List, Optional, Set, Tuple
-
+from typing import Any, Dict, Generator, List, Optional, Set, Tuple
 
 TIMEOUT_OPEN = 60  # Try and fix some of the breakages of sqlite3.
 
@@ -47,7 +46,9 @@ class KeyValueSqlite:
     database file.
     """
 
-    def __init__(self, db_path: str, table_name: str) -> None:
+    def __init__(self, db_path: str, table_name: Optional[str] = None) -> None:
+        """Initialize the database."""
+        table_name = table_name or "default"
         self.db_path = to_path(db_path)
         folder_path = os.path.dirname(self.db_path)
         os.makedirs(folder_path, exist_ok=True)
@@ -61,16 +62,14 @@ class KeyValueSqlite:
         with self.open_db_for_read() as conn:
             # Check to see if it's exists first of all.
             check_table_stmt = (
-                "SELECT name FROM sqlite_master WHERE type='table' AND name='%s';"
-                % self.table_name
+                "SELECT name FROM sqlite_master WHERE type='table' AND name='%s';" % self.table_name
             )
             cursor = conn.execute(check_table_stmt)
             has_table = cursor.fetchall()
             if has_table:
                 return
         create_stmt = (
-            "CREATE TABLE %s (key TEXT PRIMARY KEY UNIQUE NOT NULL, value TEXT);"
-            % self.table_name
+            "CREATE TABLE %s (key TEXT PRIMARY KEY UNIQUE NOT NULL, value TEXT);" % self.table_name
         )
         with self.open_db_for_write() as conn:
             try:
@@ -79,14 +78,17 @@ class KeyValueSqlite:
                 pass  # Table already created
 
     @contextmanager
-    def open_db_for_write(self):
+    def open_db_for_write(
+        self, isolation_level: Optional[str] = None, timeout: int = TIMEOUT_OPEN
+    ) -> Generator[sqlite3.Connection, None, None]:
         """Obtains an exclusive lock and does a write."""
+        isolation_level = isolation_level or "EXCLUSIVE"
         try:
             conn = sqlite3.connect(
                 self.db_path,
-                isolation_level="EXCLUSIVE",
+                isolation_level=isolation_level,
                 check_same_thread=False,
-                timeout=TIMEOUT_OPEN,
+                timeout=timeout,
             )
         except sqlite3.OperationalError as err:
             raise OSError(f"Error while opening {self.db_path}") from err
@@ -99,7 +101,7 @@ class KeyValueSqlite:
             conn.close()
 
     @contextmanager
-    def open_db_for_read(self):  # type: ignore
+    def open_db_for_read(self) -> Generator[sqlite3.Connection, None, None]:
         """
         Obtains an exclusive lock and reads. TODO: This should be a shared lock for
         concurrent read access.
@@ -107,7 +109,6 @@ class KeyValueSqlite:
         try:
             conn = sqlite3.connect(
                 self.db_path,
-                isolation_level="EXCLUSIVE",
                 check_same_thread=False,
                 timeout=60,
             )
@@ -125,9 +126,7 @@ class KeyValueSqlite:
         """
         check_key(key)
         val = json_encode(val)
-        insert_stmt = (
-            "INSERT OR IGNORE INTO %s (key, value) VALUES (?, ?)" % self.table_name
-        )
+        insert_stmt = "INSERT OR IGNORE INTO %s (key, value) VALUES (?, ?)" % self.table_name
         record = (key, val)
         with self.open_db_for_write() as conn:
             conn.execute("BEGIN")
@@ -182,9 +181,7 @@ class KeyValueSqlite:
         """Like dict.set(key) = value"""
         check_key(key)
         val = json_encode(val)
-        insert_stmt = (
-            "INSERT OR REPLACE INTO %s (key, value) VALUES (?, ?)" % self.table_name
-        )
+        insert_stmt = "INSERT OR REPLACE INTO %s (key, value) VALUES (?, ?)" % self.table_name
         record = (key, val)
         with self.open_db_for_write() as conn:
             conn.execute("BEGIN")
@@ -275,9 +272,7 @@ class KeyValueSqlite:
         """
         check_key(key_low)
         check_key(key_high)
-        select_stmt = "SELECT key, value FROM %s WHERE key BETWEEN ? AND ?" % (
-            self.table_name
-        )
+        select_stmt = "SELECT key, value FROM %s WHERE key BETWEEN ? AND ?" % (self.table_name)
         output = []
         with self.open_db_for_read() as conn:
             conn.execute("BEGIN")
@@ -295,9 +290,7 @@ class KeyValueSqlite:
         """Outputs an ordered sequence starting form key_low to key_high."""
         output: List[str, Any]  # type: ignore
         output = []  # type: ignore
-        select_stmt = (
-            f"SELECT key, value FROM {self.table_name} WHERE key BETWEEN ? AND ?"
-        )
+        select_stmt = f"SELECT key, value FROM {self.table_name} WHERE key BETWEEN ? AND ?"
         values = (key_low, key_high)
         with self.open_db_for_read() as conn:
             conn.execute("BEGIN")
@@ -334,9 +327,7 @@ class KeyValueSqlite:
         """Like dict.update()"""
         for key in a_dict:
             check_key(key)
-        insert_stmt = (
-            "INSERT OR REPLACE INTO %s (key, value) VALUES (?, ?)" % self.table_name
-        )
+        insert_stmt = "INSERT OR REPLACE INTO %s (key, value) VALUES (?, ?)" % self.table_name
         records = [(key, json_encode(val)) for key, val in a_dict.items()]
         with self.open_db_for_write() as conn:
             conn.execute("BEGIN")
@@ -361,12 +352,25 @@ class KeyValueSqlite:
         """
         for key in a_dict:
             check_key(key)
-        insert_stmt = (
-            "INSERT OR IGNORE INTO %s (key, value) VALUES (?, ?)" % self.table_name
-        )
+        insert_stmt = "INSERT OR IGNORE INTO %s (key, value) VALUES (?, ?)" % self.table_name
         data = a_dict.items()
         records = [(key, json_encode(val)) for key, val in data]
         with self.open_db_for_write() as conn:
             conn.execute("BEGIN")
             conn.executemany(insert_stmt, records)
             conn.commit()
+
+    def atomic_add(self, key: str, value: int) -> None:
+        """
+        Adds value to the value associated with key.
+        """
+        check_key(key)
+        update_stmt = f"UPDATE {self.table_name} SET value = value + ? WHERE key = ?"
+        insert_stmt = "INSERT INTO %s (key, value) VALUES (?, ?)" % self.table_name
+        values = (value, key)
+        with self.open_db_for_write(isolation_level="EXCLUSIVE") as conn:
+            conn.execute("BEGIN")
+            cursor = conn.execute(update_stmt, values)
+            if cursor.rowcount == 0:
+                conn.execute(insert_stmt, (key, value))
+            conn.commit()
